/*
 * esteid.h: OpenSC esteid header file
 *
 * Copyright (C) 2001, 2002  Juha Yrjl <juha.yrjola@iki.fi>
 * Copyright (C) 2002  Priit Randla <priit.randla@eyp.ee>
 * Copyright (C) 2002  Marie Fischer <marie@vtl.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//modified pkcs15.h by mf

#ifndef _ESTEID_H
#define _ESTEID_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <opensc/opensc.h>

#define SC_ESTEID_CACHE_DIR		".eid"

#define SC_ESTEID_PIN_MAGIC		0x31415926
#define SC_ESTEID_MAX_PINS		3		//PUK is treated like PIN
#define SC_ESTEID_MAX_PRKEYS		2
#define SC_ESTEID_MAX_PUBKEYS		2		//unchecked
#define SC_ESTEID_MAX_LABEL_SIZE	64		//orig=32
#define SC_ESTEID_MAX_ID_SIZE		16
#define SC_ESTEID_MAX_DFS		4		//unchecked
#define SC_ESTEID_MAX_CERTS		2	/* Total certificates, was 4 - mf */

#define SC_ESTEID_PIN_MAX_LEN		12		//from eyp

struct sc_esteid_id {
	u8 value[SC_ESTEID_MAX_ID_SIZE];
	size_t len;
};
typedef struct sc_esteid_id sc_esteid_id_t;

// unchecked
#define SC_ESTEID_CO_FLAG_PRIVATE	0x00000001
#define SC_ESTEID_CO_FLAG_MODIFIABLE	0x00000002
#define SC_ESTEID_CO_FLAG_OBJECT_SEEN	0x80000000 /* for PKCS #11 module */

//unchecked
#define SC_ESTEID_PIN_FLAG_CASE_SENSITIVE		0x0001
#define SC_ESTEID_PIN_FLAG_LOCAL			0x0002
#define SC_ESTEID_PIN_FLAG_CHANGE_DISABLED		0x0004
#define SC_ESTEID_PIN_FLAG_UNBLOCK_DISABLED		0x0008
#define SC_ESTEID_PIN_FLAG_INITIALIZED			0x0010
#define SC_ESTEID_PIN_FLAG_NEEDS_PADDING		0x0020
#define SC_ESTEID_PIN_FLAG_UNBLOCKING_PIN		0x0040
#define SC_ESTEID_PIN_FLAG_SO_PIN			0x0080
#define SC_ESTEID_PIN_FLAG_DISABLE_ALLOW		0x0100
#define SC_ESTEID_PIN_FLAG_INTEGRITY_PROTECTED		0x0200
#define SC_ESTEID_PIN_FLAG_CONFIDENTIALITY_PROTECTED	0x0400
#define SC_ESTEID_PIN_FLAG_EXCHANGE_REF_DATA		0x0800

//unchecked
#define SC_ESTEID_PIN_TYPE_BCD				0
#define SC_ESTEID_PIN_TYPE_ASCII_NUMERIC		1
#define SC_ESTEID_PIN_TYPE_UTF8				2

// application using the card
// this is so that pkcs11 apps (browsers) see only the first cert/prkey/pin
#define SC_ESTEID_APP_PKCS11	1
#define SC_ESTEID_APP_OTHER	0

struct sc_esteid_pin_info {
	struct sc_esteid_id auth_id;
	int reference;
	int flags, type;
	int min_length, stored_length;
	u8 pad_char;
	struct sc_path path;
	int tries_left;
	int verified;			//from eyp
	unsigned int magic;
};
typedef struct sc_esteid_pin_info sc_esteid_pin_info;

#define SC_ESTEID_ALGO_OP_COMPUTE_CHECKSUM	0x01
#define SC_ESTEID_ALGO_OP_COMPUTE_SIGNATURE	0x02
#define SC_ESTEID_ALGO_OP_VERIFY_CHECKSUM	0x04
#define SC_ESTEID_ALGO_OP_VERIFY_SIGNATURE	0x08
#define SC_ESTEID_ALGO_OP_ENCIPHER		0x10
#define SC_ESTEID_ALGO_OP_DECIPHER		0x20
#define SC_ESTEID_ALGO_OP_HASH			0x40
#define SC_ESTEID_ALGO_OP_GENERATE_KEY		0x80

struct sc_esteid_algorithm_info {
	int reference;
	int algorithm, supported_operations;
};

/* A large integer, big endian notation */
struct sc_esteid_bignum {
	u8 *		data;
	size_t		len;
};
typedef struct sc_esteid_bignum sc_esteid_bignum_t;

struct sc_esteid_der {
	u8 *		value;
	size_t		len;
};
typedef struct sc_esteid_der sc_esteid_der_t;

struct sc_esteid_pubkey_rsa {
	sc_esteid_bignum_t modulus;
	sc_esteid_bignum_t exponent;
};

struct sc_esteid_prkey_rsa {
	/* public components */
	sc_esteid_bignum_t modulus;
	sc_esteid_bignum_t exponent;

	/* private components */
	sc_esteid_bignum_t d;
	sc_esteid_bignum_t p;
	sc_esteid_bignum_t q;

	/* optional CRT elements */
	sc_esteid_bignum_t iqmp;
	sc_esteid_bignum_t dmp1;
	sc_esteid_bignum_t dmq1;
};

struct sc_esteid_pubkey_dsa {
	sc_esteid_bignum_t pub;
	sc_esteid_bignum_t p;
	sc_esteid_bignum_t q;
	sc_esteid_bignum_t g;
};

struct sc_esteid_prkey_dsa {
	/* public components */
	sc_esteid_bignum_t pub;
	sc_esteid_bignum_t p;
	sc_esteid_bignum_t q;
	sc_esteid_bignum_t g;

	/* private key */
	sc_esteid_bignum_t priv;
};

struct sc_esteid_pubkey {
	int algorithm;

	/* Decoded key */
	union {
		struct sc_esteid_pubkey_rsa rsa;
		struct sc_esteid_pubkey_dsa dsa;
	} u;

	/* DER encoded raw key */
	sc_esteid_der_t data;
};
typedef struct sc_esteid_pubkey sc_esteid_pubkey_t;

//unchecked
struct sc_esteid_prkey {
	int algorithm;
	union {
		struct sc_esteid_prkey_rsa rsa;
		struct sc_esteid_prkey_dsa dsa;
	} u;
};
typedef struct sc_esteid_prkey sc_esteid_prkey_t;

/* Enveloped objects can be used to provide additional
 * protection to non-native private keys */
//unchecked
struct sc_esteid_enveloped_data {
	/* recipient info */
	sc_esteid_id_t id;		/* key ID */
	struct sc_algorithm_id ke_alg;	/* key-encryption algo */
	u8 *key;			/* encrypted key */
	size_t key_len;
	struct sc_algorithm_id ce_alg;	/* content-encryption algo */
	u8 *content;			/* encrypted content */
	size_t content_len;
};

struct sc_esteid_cert {
	int version;
	u8 *serial;
	size_t serial_len;
	u8 *issuer;
	size_t issuer_len;
	u8 *subject;
	size_t subject_len;
	u8 *crl;
	size_t crl_len;

	struct sc_esteid_pubkey key;
	u8 *data;	/* DER encoded raw cert */
	size_t data_len;
};
typedef struct sc_esteid_cert sc_esteid_cert_t;

struct sc_esteid_cert_info {
	char label[SC_ESTEID_MAX_LABEL_SIZE];		//from eyp
	struct sc_esteid_id id;	/* correlates to private key id */
	int authority;		/* boolean */
	/* identifiers [2] SEQUENCE OF CredentialIdentifier{{KeyIdentifiers}} */
	struct sc_path path;
};
typedef struct sc_esteid_cert_info sc_esteid_cert_info_t;

#define SC_ESTEID_PRKEY_USAGE_ENCRYPT		0x01
#define SC_ESTEID_PRKEY_USAGE_DECRYPT		0x02
#define SC_ESTEID_PRKEY_USAGE_SIGN		0x04
#define SC_ESTEID_PRKEY_USAGE_SIGNRECOVER	0x08
#define SC_ESTEID_PRKEY_USAGE_WRAP		0x10
#define SC_ESTEID_PRKEY_USAGE_UNWRAP		0x20
#define SC_ESTEID_PRKEY_USAGE_VERIFY		0x40
#define SC_ESTEID_PRKEY_USAGE_VERIFYRECOVER	0x80
#define SC_ESTEID_PRKEY_USAGE_DERIVE		0x100
#define SC_ESTEID_PRKEY_USAGE_NONREPUDIATION	0x200

#define SC_ESTEID_PRKEY_ACCESS_SENSITIVE	0x01
#define SC_ESTEID_PRKEY_ACCESS_EXTRACTABLE	0x02
#define SC_ESTEID_PRKEY_ACCESS_ALWAYSSENSITIVE	0x04
#define SC_ESTEID_PRKEY_ACCESS_NEVEREXTRACTABLE	0x08
#define SC_ESTEID_PRKEY_ACCESS_LOCAL		0x10

#define SC_ESTEID_PUBLIC_LINE_COUNT		16		//from eyp
#define SC_ESTEID_PUBLIC_MAXLINELEN		50		//from eyp

struct sc_esteid_prkey_info {
	char label[SC_ESTEID_MAX_LABEL_SIZE];			//from eyp
	struct sc_esteid_id id;	/* correlates to public certificate id */
	unsigned int usage, access_flags;
	int native, key_reference;
	int modulus_length;

	struct sc_path path;
};
typedef struct sc_esteid_prkey_info sc_esteid_prkey_info;

struct sc_esteid_pubkey_info {
	char label[SC_ESTEID_MAX_LABEL_SIZE];			//from eyp
	struct sc_esteid_id id;	/* correlates to private key id */
	unsigned int usage, access_flags;
	int native, key_reference;
	int modulus_length;

	struct sc_path path;
};
typedef struct sc_esteid_pubkey_info sc_esteid_pubkey_info;

//unchecked
#define SC_ESTEID_TYPE_CLASS_MASK		0xF00

//unchecked
#define SC_ESTEID_TYPE_PRKEY			0x100
#define SC_ESTEID_TYPE_PRKEY_RSA		0x101
#define SC_ESTEID_TYPE_PRKEY_DSA		0x102

#define SC_ESTEID_TYPE_PUBKEY			0x200
#define SC_ESTEID_TYPE_PUBKEY_RSA		0x201
#define SC_ESTEID_TYPE_PUBKEY_DSA		0x202

#define SC_ESTEID_TYPE_CERT			0x400
#define SC_ESTEID_TYPE_CERT_X509		0x401
#define SC_ESTEID_TYPE_CERT_SPKI		0x402

#define SC_ESTEID_TYPE_DATA_OBJECT		0x500
#define SC_ESTEID_TYPE_AUTH			0x600
#define SC_ESTEID_TYPE_AUTH_PIN			0x601


struct sc_esteid_object {
	int type;
	// CommonObjectAttributes 
	char label[SC_ESTEID_MAX_LABEL_SIZE];	//zero terminated
	int flags;
	struct sc_esteid_id auth_id;

	int user_consent;

	// Object type specific data
	void *data;

	struct sc_esteid_df *df; // can be NULL, if object is 'floating'
	struct sc_esteid_object *next, *prev; // used only internally
};
typedef struct sc_esteid_object sc_esteid_object_t;



/* PKCS #15 DF types */
// probably won't need them..?
#define SC_ESTEID_PRKDF			0
#define SC_ESTEID_PUKDF			1
#define SC_ESTEID_PUKDF_TRUSTED		2
#define SC_ESTEID_SKDF			3
#define SC_ESTEID_CDF			4
#define SC_ESTEID_CDF_TRUSTED		5
#define SC_ESTEID_CDF_USEFUL		6
#define SC_ESTEID_DODF			7
#define SC_ESTEID_AODF			8
#define SC_ESTEID_DF_TYPE_COUNT		9

//unchecked
struct sc_esteid_df {
	struct sc_file *file;

	struct sc_path path;
	int record_length, type;
	int enumerated;

	struct sc_esteid_df *next, *prev;
};

#define SC_ESTEID_CARD_MAGIC		0x10203040

typedef struct sc_esteid_card {
	sc_card_t *card;
	char *label;
	char *public_info[SC_ESTEID_PUBLIC_LINE_COUNT];		//from eyp
	/* fields from TokenInfo: */
	int version;
	char *serial_number, *manufacturer_id;
	unsigned long flags;
	struct sc_esteid_algorithm_info alg_info[1];


	struct sc_esteid_object *certs[SC_ESTEID_MAX_CERTS];
	int cert_count;
	struct sc_esteid_object *prkeys[SC_ESTEID_MAX_PRKEYS];
	int prkey_count;
	struct sc_esteid_object *pins[SC_ESTEID_MAX_PINS];
	int pin_count;
	
	struct sc_esteid_card_opts {
		int use_cache;
	} opts;

	unsigned int magic;
} sc_esteid_card_t;

//from eyp
#define SURNAME		0
#define GIVEN_NAMES1	1
#define GIVEN_NAMES2	2
#define SEX		3
#define CITIZENSHIP	4
#define DATE_OF_BIRTH	5
#define PERSONAL_ID	6
#define DOCUMENT_NR	7
#define EXPIRY_DATE	8
#define PLACE_OF_BIRTH	9
#define ISSUING_DATE	10
#define PERMIT_TYPE	11
#define REMARK1		12
#define REMARK2		13
#define REMARK3		14
#define REMARK4		15


#define SC_ESTEID_CARD_FLAG_READONLY		0x01
#define SC_ESTEID_CARD_FLAG_LOGIN_REQUIRED	0x02
#define SC_ESTEID_CARD_FLAG_PRN_GENERATION	0x04
#define SC_ESTEID_CARD_FLAG_EID_COMPLIANT	0x08

//from eyp
#define SC_ERROR_ESTEID_NOT_FOUND		-1007

//from eyp
struct sc_esteid_defaults {
	const char *ef_dir_dump;
	int (*defaults_func)(struct sc_esteid_card *, int arg);
	int arg;
};

//from eyp
//select 3500/eeee
int select_esteid_df(struct sc_esteid_card *card);

//from eyp
//prepare signature operation
int esteid_prepare_sign(struct sc_esteid_card *p15card, const struct sc_esteid_prkey_info *prkey, int hash);

int sc_esteid_bind(struct sc_card *card,
		   struct sc_esteid_card **esteid_card);

int sc_esteid_unbind(struct sc_esteid_card *card);


struct sc_esteid_card * sc_esteid_card_new();

void sc_esteid_card_free(struct sc_esteid_card *p15card);

void sc_esteid_print_card(const struct sc_esteid_card *card);


//int esteid_prepare_decipher( struct sc_esteid_card *p15card, 
//				    const struct sc_esteid_prkey_info *prkey)
int sc_esteid_decipher(struct sc_esteid_card *p15card,
//		       const struct sc_esteid_object *prkey_obj,
//		       unsigned long flags,
//		       const u8 *in, size_t inlen, u8 *out, size_t outlen);
				const struct sc_esteid_prkey_info *prkey,
				const u8 * in, int inlen, u8 *out, int outlen);

int sc_esteid_compute_signature(struct sc_esteid_card *p15card,
//				const struct sc_esteid_object *prkey_obj,
//				unsigned long alg_flags, const u8 *in,
//				size_t inlen, u8 *out, size_t outlen);
				const struct sc_esteid_prkey_info *prkey,
				int hash, const u8 *in, int inlen, u8 *out,
				int outlen);


//from eyp
#define SC_ESTEID_HASH_NONE	0
#define SC_ESTEID_HASH_SHA1	1



//pubkey
int sc_esteid_decode_pubkey_rsa(struct sc_context *ctx,
	       		struct sc_esteid_pubkey_rsa *pubkey,
			const u8 *, size_t);
int sc_esteid_decode_pubkey_dsa(struct sc_context *ctx,
	       		struct sc_esteid_pubkey_dsa *pubkey,
			const u8 *, size_t);
int sc_esteid_decode_pubkey(struct sc_context *,
	       		struct sc_esteid_pubkey *, const u8 *, size_t);
void sc_esteid_erase_pubkey(struct sc_esteid_pubkey *pubkey);

// cert
int sc_esteid_enum_certificates(struct sc_esteid_card *card, int application);
int sc_esteid_read_certificate(struct sc_esteid_card *card,
			       const struct sc_esteid_cert_info *info,
			       struct sc_esteid_cert **cert);
void sc_esteid_free_certificate(struct sc_esteid_cert *cert);
int sc_esteid_find_cert_by_id(struct sc_esteid_card *card,
			      const struct sc_esteid_id *id,
			      struct sc_esteid_object **out);
//			      struct sc_esteid_cert_info **cert_out);

// prkey
int sc_esteid_enum_private_keys(struct sc_esteid_card *card, int application);
int sc_esteid_find_prkey_by_id(struct sc_esteid_card *card,
			       const struct sc_esteid_id *id,
			       struct sc_esteid_object **out);
//				struct sc_esteid_prkey_info **key_out);
			       
// pin
int sc_esteid_verify_pin(struct sc_esteid_card *card,
			 struct sc_esteid_pin_info *pin,
			 const u8 *pincode, size_t pinlen);
int sc_esteid_change_pin(struct sc_esteid_card *card,
			 struct sc_esteid_pin_info *pin,
			 const u8 *oldpincode, size_t oldpinlen,
			 const u8 *newpincode, size_t newpinlen);
int sc_esteid_find_pin_by_auth_id(struct sc_esteid_card *card,
				  const struct sc_esteid_id *id,
				  struct sc_esteid_object **out);
//					struct sc_esteid_pin_info **pin_out);
int sc_esteid_enum_pins(struct sc_esteid_card *p15card, int application);


/* Generic file i/o */
int sc_esteid_read_file(struct sc_esteid_card *p15card,
			const struct sc_path *path,
			u8 **buf, size_t *buflen,
			struct sc_file **file_out);

/* Caching functions */
int sc_esteid_read_cached_file(struct sc_esteid_card *p15card,
                               const struct sc_path *path,
                               u8 **buf, size_t *bufsize);
int sc_esteid_cache_file(struct sc_esteid_card *p15card,
			 const struct sc_path *path,
			 const u8 *buf, size_t bufsize);

/* EstEID ID handling functions */
int sc_esteid_compare_id(const struct sc_esteid_id *id1,
			 const struct sc_esteid_id *id2);
void sc_esteid_print_id(const struct sc_esteid_id *id);
void sc_esteid_format_id(const char *id_in, struct sc_esteid_id *id_out);
int sc_esteid_hex_string_to_id(const char *in, struct sc_esteid_id *out);

int sc_esteid_get_key_usage(struct sc_esteid_card *estcard, int rec);

int sc_esteid_unblock_pin(struct sc_esteid_card *estcard,
			 struct sc_esteid_pin_info *puk,
			 const u8 *pukcode, size_t puklen, int ref);


int sc_esteid_unblock_and_change_pin(struct sc_esteid_card *estcard,
			 struct sc_esteid_pin_info *puk,
			 const u8 *pukcode, size_t puklen,
			 struct sc_esteid_pin_info *newpin,
			 const u8 *newpincode, size_t newpinlen);


#ifdef  __cplusplus
}
#endif

#endif
